/*-
 * Copyright (c) 1999-2003 Nick Johnson
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id$
 */

#include <unistd.h>
#include <fcntl.h>
#include <kvm.h>
#include <stdio.h>
#include <string.h>
#include <sys/vmmeter.h>

#define pagetok(size) ((size) << pgshift)	/* from top's machine.c */

int             getkval(kvm_t *, unsigned long, int *, int, char *);
void            usage(void);

extern int      errno;

void
usage(void)
{
	printf("Usage: muse [-k|-m]\n");
}

int
main(argc, argv)
	int             argc;
	char          **argv;

{
	int             pgsz, pgshift;
	kvm_t          *kdesc;
	struct vmmeter  vmstats;
	unsigned long   cnt, buf;
	unsigned int    buffer;
	float           kdiv;
	char           *units;
	int             format = 0;

	struct nlist    namelist[] =
	{
		{"_bufspace"},
		{"_cnt"},
		{0}
	};

	if (argc == 2) {
		if (!strcmp(argv[1], "-k")) {
			kdiv = 1024;
			units = "kB";
			format = 0;
		} else if (!strcmp(argv[1], "-m")) {
			kdiv = 1048576;
			units = "MB";
			format = 3;
		} else {
			usage();
			return (-1);
		}
	} else if (argc > 2) {
		usage();
		return (-1);
	} else {
		kdiv = 1;
		units = "Bytes";
	}

	pgsz = getpagesize();
	pgshift = 0;
	while (pgsz > 1) {
		pgshift++;
		pgsz >>= 1;
	}


	if ((kdesc = kvm_open(NULL, NULL, NULL, O_RDONLY, "kvm_open")) == NULL) {
		fprintf(stderr, "kvm_open failed");
		return (-1);
	}
	(void)kvm_nlist(kdesc, namelist);

	if (namelist[0].n_type == 0 || namelist[1].n_type == 0) {
		fprintf(stderr, "kvm_nlist failed");
		return (-1);
	}
	buf = namelist[0].n_value;
	cnt = namelist[1].n_value;

	(void)getkval(kdesc, cnt, (int *)(&vmstats), sizeof(vmstats), "_cnt");
	(void)getkval(kdesc, buf, (int *)(&buffer), sizeof(buffer), "_bufspace");

	printf("Active:   %10.*f %s\n", format, pagetok(vmstats.v_active_count) / kdiv, units);
	printf("Inactive: %10.*f %s\n", format, pagetok(vmstats.v_inactive_count) / kdiv, units);
	printf("Wired:    %10.*f %s\n", format, pagetok(vmstats.v_wire_count) / kdiv, units);
	printf("Reserved: %10.*f %s\n", format, pagetok(vmstats.v_free_reserved) / kdiv, units);
	printf("Cache:    %10.*f %s\n", format, pagetok(vmstats.v_cache_count) / kdiv, units);
	printf("Kernel:   %10.*f %s\n", format, pagetok(vmstats.v_pageout_free_min) / kdiv, units);
	printf("Interrupt:%10.*f %s\n", format, pagetok(vmstats.v_interrupt_free_min) / kdiv, units);

	printf("Buffer:   %10.*f %s\n\n", format, buffer / kdiv, units);

	printf("Total:    %10.*f %s\n", format, pagetok(vmstats.v_page_count) / kdiv, units);
	printf("Free:     %10.*f %s\n", format, pagetok(vmstats.v_free_count) / kdiv, units);

	return (0);
}

/* stolen right out of top's machine.c, with "kd" added as a parameter */

int
getkval(kd, offset, ptr, size, refstr)
	kvm_t          *kd;
	unsigned long   offset;
	int            *ptr;
	int             size;
	char           *refstr;

{
	if (kvm_read(kd, offset, (char *)ptr, size) != size) {
		if (*refstr == '!') {
			return (0);
		} else {
			fprintf(stderr, "muse: kvm_read for %s: %s\n",
				refstr, strerror(errno));
			exit(-1);
		}
	}
	return (1);
}
